C     Copyright (C) 2004 B. Roukema et al. <shape-univ at astro.uni.torun.pl>
C     
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C     
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C     
C     See also http://www.gnu.org/licenses/gpl.html

      subroutine rddata(doCOBE,doWMAP,
     _     smooth,new,sigdeg,resdeg,
     _     dosmap,dos2map,dos2mean, 
     _     indir2,nindir,
     _     galcut,bcut,
     _     Ssigma)
C     INPUTS:
c     ---- smoothing ? ----
      logical*4 doCOBE          ! read in COBE data?
      logical*4 doWMAP          ! read in WMAP data?
      logical*4 smooth          ! is either a new or old smoothed map required?
      logical*4 new             ! do a new smoothing?

c     if(smooth)then
c     new=.true  means a new smoothed map will be calculated
c     With version 2004-01-05.1 or earlier, this is slow and inefficient,
c     but correct, taking about an hour or so on a 1.7GHz machine.
c     
c     new=.false.  means a previously calculated smoothed map will be read 
c     
c     elseif(.not.(smooth))then
c     the original unsmoothed data will be read in and used
c     endif

      real*4    sigdeg,resdeg   ! maximum sigma, resolution for smoothing (deg)

      logical*4 dosmap,dos2map,dos2mean ! calculating S type statistics


      character*256  indir2    ! directory with input files
      integer        nindir    ! number of characters in  indir

C     INPUTS USED FOR TESTING ONLY:
      logical*4 galcut ! is the galactic plane removed?
      real*4    bcut            ! galactic latitude cut


C     OUTPUTS:
      real*4   Ssigma

C     COMMON:

      include 'npix.dec'
      real*4   temp(maxn),errtemp(maxn) !   Temperature and uncert.
c     real*4   eclong(maxn),eclat(maxn) ! ecliptic
      real*4   gallong(maxn),gallat(maxn) ! galactic
      real*4   eqlong(maxn),eqlat(maxn) ! ra dec in degrees, degrees

      real*4   xg(maxn),yg(maxn),zg(maxn) ! galactic x,y,z for unit radius

c     parameter(nxint=180,nyint=90)
      include 'nxint.dec'
      integer*4 inear(nxint,nyint) ! nearest pixel number
      real*4    dinear(nxint,nyint)
      real*4    dxint,dyint
c     -- Eq. Cartesian coords of table --
      real*4    x(nxint,nyint),y(nxint,nyint),z(nxint,nyint) 

      common /cobecomm/  temp,errtemp,gallong,gallat,eqlong,eqlat,
     _     dsm,gb0,dxint,dyint,inear,dinear,x,y,z
      

      logical*4 measur,noise    
c     real*4    ranerr(maxn)
      common /noisecomm/ measur,noise

C     LOCAL:

C     :END OF DECLARATIONS

      measur=.true.
      noise=.false.

c     if(doCOBE)then
c     smooth=.true.    !.false. !
c     else
c     smooth=.false.
c     endif



c     ---- the original unsmoothed map is needed for error estimates ----
c     if(.not.(smooth.and.(.not.new)))then

      print*,'doCOBE=',doCOBE
      if(doCOBE)then
         print*,'Will call rdCOBE.'
         print*,'rdCOBE disabled because fortran is'//
     _        ' inefficient in memory management.'
         print*,'Please change npix.dec, uncomment '//
     _        'the option in circleT2.f, and recompile.'
c     call rdCOBE
         print*,'Have called rdCOBE.'
      elseif(doWMAP)then
         print*,'Will call rdWMAP.'
         call rdWMAP(indir2,nindir)
         print*,'Have called rdWMAP.'
      else
         print*,'WARNING: no cmb file read in.'
      endif

c     endif
      
      if(smooth)then
         if(doCOBE)then
            call 
     _           smoothCOBE(new,
     _           dosmap,dos2map,do2smean,angledeg)
            print*,'from file read in, angledeg=',angledeg

         elseif(doWMAP)then
            call smooWMAP(new,sigdeg,resdeg, indir2,nindir)
            print*,'from file read in, sigdeg,resdeg=',
     _           sigdeg,resdeg

         else
            stop 'if(smooth): no other options'
         endif
      endif
      

c     ---- statistics of COBE signal S and noise B ("B" for bruit)----
      Smean=0.0
      Ssigma=0.0
      Bmean=0.0
      Bsigma=0.0
      do i=1,npix
         if(.not.(galcut).or.abs(gallat(i)).gt.bcut)then
            Smean=Smean+temp(i)
            Bmean=Bmean+errtemp(i)
         endif
      enddo
      Smean=Smean/real(npix)
      Bmean=Bmean/real(npix)


      do i=1,npix
         if(.not.(galcut).or.abs(gallat(i)).gt.bcut)then
            ss=temp(i)- Smean
            Ssigma=Ssigma+ ss*ss
            ss= errtemp(i) - Bmean
            Bsigma=Bsigma+ ss*ss
         endif
      enddo
      Ssigma=sqrt(Ssigma/real(npix-1))
      Bsigma=sqrt(Bsigma/real(npix-1))

      print*,'signal: Smean,Ssigma=',Smean,Ssigma
      print*,'noise: Bmean,Bsigma=',Bmean,Bsigma
      print*,'Ssigma/Bmean=',Ssigma/Bmean

c     cutnoise= factcutnoise * Ssigma !Bmean
c     print*,'cutnoise= ',cutnoise

      return
      end


      
